#include <math.h>
#include "Component.h"

CComponent::CComponent(){};
CComponent::~CComponent(){};

//================================================
CNaChannel::CNaChannel()
{
	neqs = 2;
	vtraub = -55.0; 
}

CNaChannel::~CNaChannel(){}

void CNaChannel::initState(double* y)
{
	double a,b;
	double v2 = y[0]-vtraub;

	a = 0.32 * (13-v2) / ( exp((13-v2)/4) - 1);
	b = 0.28 * (v2-40) / ( exp((v2-40)/5) - 1);
	y[bp+0]=a/(a+b);

	a = 0.128 * exp((17-v2)/18);
	b = 4 / ( 1 + exp((40-v2)/5) );
	y[bp+1]=a/(a+b);
}

void CNaChannel::rhs(double x, double*y, double* dy)
{
	double v2 = y[0]-vtraub;
	double a,b;
// m-gate
	a = 0.32 * (13-v2) / ( exp((13-v2)/4) - 1);
	b = 0.28 * (v2-40) / ( exp((v2-40)/5) - 1);
	dy[bp+0]= a - (a+b)*y[bp+0];
//h-gate
	a = 0.128 * exp((17-v2)/18);
	b = 4 / ( 1 + exp((40-v2)/5) );
	dy[bp+1] = a - (a+b)*y[bp+1];
}

//================================================

CKChannel::CKChannel()
{
	neqs = 1;
	vtraub = -55.0; 
}
CKChannel::~CKChannel(){}

void CKChannel::initState(double* y)
{
	double v2 = y[0]-vtraub;
	double a,b;
// n-gate
	a = 0.032 * (15-v2) / ( exp((15-v2)/5) - 1);
	b = 0.5 * exp((10-v2)/40);
	y[bp+0]= a/(a+b);
}

void CKChannel::rhs(double x, double*y, double* dy)
{
	double v2 = y[0]-vtraub;
	double a,b;
// n-gate
	a = 0.032 * (15-v2) / ( exp((15-v2)/5) - 1);
	b = 0.5 * exp((10-v2)/40);
	dy[bp+0]= a - (a+b)*y[bp+0];
}

//================================================

CTCaChannel::CTCaChannel()
{
	neqs = 2;
}

CTCaChannel::~CTCaChannel(){}

void CTCaChannel::initState(double* y)
{
	double vshift=0;
	double vs = y[0] + vshift;

	y[bp+0] = 1.0 / ( 1 + exp(-(vs+50)/7.4) );
	y[bp+1] = 1.0 / ( 1 + exp((vs+78)/5.0) );
}

void CTCaChannel::rhs(double x, double*y, double* dy)
{
	double vshift=0;
	double vs = y[0] + vshift;
	double tau, inf;

	tau = ( 3 + 1.0 / ( exp((vs+25)/10) + exp(-(vs+100)/15) ) ); 
	inf = 1.0 / ( 1 + exp(-(vs+50)/7.4) );
    dy[bp+0] = (inf - y[bp+0]) / tau;

	tau = ( 85 + 1.0 / ( exp((vs+46)/4) + exp(-(vs+405)/50) ) ) ;
	inf = 1.0 / ( 1 + exp((vs+78)/5.0) );
    dy[bp+1] = (inf - y[bp+1]) / tau;
}

